/* $Id: trafdisp.c,v 1.4 2001/03/19 18:15:25 tcg Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "trafdisp.h"

extern char *interface; /* Interface to monitor, defined in io-*.c */
extern char hostname[ BUFSIZ ]; /* The name of the hostname, defined in log.c */
int nofork = 0; /* Should we stay on foreground? */
int interval = 60; /* Number of seconds between each sample */

/* Prototypes */
void help( );
void event_loop( );
int loginit( ); /* from log.c */
int log( u_long, u_long ); /* from log.c */

int main( int argc, char *argv[] )
{
  int c;
  memset( hostname, 0, sizeof( hostname ) );
  while( ( c = getopt( argc, argv, "ohfn:i:" ) ) != -1 )
    {
      switch( c )
	{
	case 'h':
	  help( );
	  return 0;
	case 'f':
	  nofork = 1;
	  break;
	case 'n':
	  interval = atoi( optarg );
	  break;
	case 'i':
	  interface = strdup( optarg );
	  break;
	case 'o':
	  strncpy( hostname, optarg, sizeof( hostname ) );
	  break;
	}
    }
  if( !interface )
    {
      fprintf( stderr, "Warning: Using default interface\n" );
    }
  if( init_kvm( ) )
    {
      return 1;
    }
  if( loginit( ) )
    {
      fprintf( stderr, "Unable to initialize logger.\n" );
      return 1;
    }
  if( !nofork )
    {
      if( !daemon( 0, 0 ) == -1 )
	{
	  fprintf( stderr, "Unable to fork.\n" );
	  return 0;
	}
    }
  event_loop( );
  return 0;
}

void help( )
{
  printf( "Traffic Display, by Eran Rundstein <e@keves.org>\n\n" );
  printf( "Usage:\n" );
  printf( "\t-h\tShow this help screen\n" );
  printf( "\t-f\tDo not fork and print debug messages\n" );
  printf( "\t-i\tSpecify which interface to use\n" );
  printf( "\t-n\tSpecify the number of seconds to wait between each sample\n" );
  printf( "\t-o\tSpecify the hostname name.\n" );
  printf( "\nPlease contact the author incase of a problem.\n" );
  return ;
}

void event_loop( )
{
  struct io_stat old, new;
  u_long in, out;
  if( intpr( &old ) )
    {
      fprintf( stderr, "Unable to read kernel memory :(\n" );
      exit( 1 );
    }
  while( 1 )
    {
      sleep( interval );
      intpr( &new );
      in = (new.ibytes - old.ibytes) / interval;
      out = (new.obytes - old.obytes) / interval;
      if( nofork )
	{
	  printf( "Incoming bytes/sec: %lu - Outgoing bytes/sec: %lu\n", in, out );
	}
      log( in, out );
      memcpy( &old, &new, sizeof( struct io_stat ) );
    }
}
